//*---------------------------------------------------------------------------*\
//  =========                 |
//  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
//   \\    /   O peration     |
//    \\  /    A nd           | Copyright (C) 2011-2012 OpenFOAM Foundation
//     \\/     M anipulation  |
//-------------------------------------------------------------------------------

//  License
//    This file is part of OpenFOAM.
//    OpenFOAM is free software: you can redistribute it and/or modify it
//    under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
//    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
//    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
//    for more details.
//    You should have received a copy of the GNU General Public License
//    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

//  Application
    
//  \*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "OFstream.H"

// --- Functions to calculate the time derivatives of eta and Ua


              volVectorField CalculateRHSU( volVectorField Uat, volScalarField eta, volVectorField Ua, dimensionedScalar g, dimensionedScalar h, dimensionedScalar za, dimensionedScalar timeDerCoef )
              {
                       dimensionedScalar C("C", dimensionSet(0,0,0,0,0,0,0), 1);
                       
                                                                     
                       solve( fvm::Sp(C,Uat) + (sqr(za)/2 + za*h)*fvm::laplacian(Uat) == - 0.5*fvc::grad(Ua & Ua)                                       
                                                                                         - g*fvc::grad(eta) 
                                                                                         - (sqr(za)/2 + za*h)*fvc::grad(fvc::div(Uat))  
                                                                                         + (sqr(za)/2 + za*h)*fvc::laplacian(Uat) 
                                                              
                            );                     
  
                       volVectorField dUadt ("dUadt", Uat);
 
                  return dUadt;
              }     

            
             volScalarField CalculateRHSeta( volScalarField eta, volVectorField Ua, dimensionedScalar za, dimensionedScalar h, dimensionedScalar   timeDerCoef )
              {
                dimensionedScalar C1("C1", dimensionSet(0,0,-1,0,0,0,0), 0);
                volScalarField etat ("etat", C1*eta );
                
         
                solve
                ( fvm::Sp(timeDerCoef,etat) == - fvc::div( (h + eta)*Ua + h*(sqr(za)/2 - sqr(h)/6)*fvc::grad(fvc::div(Ua))               
                                                                      + h*(za + h/2)*fvc::grad(h*fvc::div(Ua)) )
                                                       
                );


                return etat;
              }






// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{
    #include "setRootCase.H"
    #include "createTime.H"
    #include "createMesh.H"
    #include "readGravitationalAcceleration.H"
    #include "readCoefficientsAndConstants.H"
    #include "createFields.H"
    
    
    

    // * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //


   int countRK;

   float rk4a[5] = { 0.0,   -567301805773.0/1357537059087.0,  -2404267990393.0/2016746695238.0, -3550918686646.0/2091501179385.0, -1275806237668.0/842570457699.0};
   float rk4b[5] = { 1432997174477.0/9575080441755.0, 5161836677717.0/13612068292357.0, 1720146321549.0/2090206949498.0, 3134564353537.0/4481467310338.0,             
                     2277821191437.0/14882151754819.0 };


   volVectorField UaRes("UaRes", Ua*0); 
   volScalarField etaRes("etaRes", eta*0);

   Info<< "\n Starting time loop\n" << endl;

    
    fileName outputFile("Elevation.txt");
    OFstream os(runTime.path()/outputFile);    
    os << "time series of midpoint\n"; 
    os << endl;

    while (runTime.loop())
    {        
    
      Info<< "\n Time = " << runTime.timeName() << nl << endl;

      dimensionedScalar dt = runTime.deltaT();

      UaRes = UaRes*0;
      etaRes = etaRes*0;

      
                // 4th order Runge Kutta loop....//

                     countRK = 0;
                                          
                     for (countRK=0; countRK<5; countRK++)
                         {
                           
                           UaRes = rk4a[countRK]*UaRes;
                           etaRes = rk4a[countRK]*etaRes;
                        
                           UaRes = UaRes + dt*CalculateRHSU( Uat, eta,  Ua, g, h, za, timeDerCoef);
                           etaRes = etaRes + dt*CalculateRHSeta( eta, Ua, za, h, timeDerCoef ); 
                           
                           Ua = Ua + rk4b[countRK]*UaRes;
                           eta = eta + rk4b[countRK]*etaRes;

                           eta.correctBoundaryConditions();      
                           Ua.correctBoundaryConditions();
                          }
 
               
        int midPoint = mesh.findCell(point(5.0,5.0,0.05));       
        
        double elevation = eta[midPoint];
        
        
        //OFstream os(runTime.path()/outputFile);
        os <<  elevation; 
        os << nl << endl;


        runTime.write();

        Info<< "ExecutionTime = " << runTime.elapsedCpuTime() << " s"
            << "  ClockTime = " << runTime.elapsedClockTime() << " s"
            << nl << endl;
    }

    Info<< "End\n" << endl;

    return 0;
}































